import { world, system } from '@minecraft/server';

const ENTITY_ID = 'la:mafangs';
const COOLDOWN_TIME = { punchbarrage: 10, deathstrike: 55, ground_impact: 20, sovereignimpale: 20, sovereignmeteor: 75, cataclysmic_blow: 100 };
const TELEPORT_COOLDOWN = 100;

const cooldowns = new Map();
let currentTarget = null;
let lastTeleportTick = 0;

function isOnCooldown(entityId, action) {
    const now = system.currentTick;
    const lastUsed = cooldowns.get(`${entityId}:${action}`) || 0;
    return now - lastUsed < COOLDOWN_TIME[action] * 20;
}

function setCooldown(entityId, action) {
    cooldowns.set(`${entityId}:${action}`, system.currentTick);
}

function teleportToTarget(entity, target) {
    if (target && target.isValid()) {
        const targetLocation = target.location;
        entity.runCommand(`particle la:dash_particle ${entity.location.x} ${entity.location.y} ${entity.location.z}`);
        entity.runCommand(`particle la:sovereignaure_charge ${entity.location.x} ${entity.location.y} ${entity.location.z}`);
        entity.teleport(targetLocation, { dimension: target.dimension });
        entity.runCommand(`particle la:dash_particle ${targetLocation.x} ${targetLocation.y} ${targetLocation.z}`);
        entity.runCommand(`particle la:sovereignaure_charge ${targetLocation.x} ${targetLocation.y} ${targetLocation.z}`);
    }
}

world.afterEvents.entityHitEntity.subscribe((data) => {
    const { damagingEntity, hitEntity } = data;

    if (damagingEntity.typeId !== ENTITY_ID) return;

    if (currentTarget && currentTarget.isValid()) {
        currentTarget.removeTag('mafangs_target');
    }
    hitEntity.addTag('mafangs_target');
    currentTarget = hitEntity;

    if (Math.random() < 0.9 && !isOnCooldown(damagingEntity.id, 'punchbarrage')) {
        damagingEntity.addTag('punchbarrage');
        setCooldown(damagingEntity.id, 'punchbarrage');
    }
    
    if (Math.random() < 0.05 && !isOnCooldown(damagingEntity.id, 'cataclysmic_blow')) {
        damagingEntity.addTag('cataclysmic_blow');
        setCooldown(damagingEntity.id, 'cataclysmic_blow');
    }

    if (Math.random() < 0.6 && !isOnCooldown(damagingEntity.id, 'sovereignimpale')) {
        damagingEntity.addTag('sovereignimpale');
        setCooldown(damagingEntity.id, 'sovereignimpale');
    }

    if (Math.random() < 0.1 && !isOnCooldown(damagingEntity.id, 'deathstrike')) {
        damagingEntity.addTag('deathstrike');
        setCooldown(damagingEntity.id, 'deathstrike');
    }

    if (Math.random() < 0.3 && !isOnCooldown(damagingEntity.id, 'sovereignmeteor')) {
        damagingEntity.addTag('sovereignmeteor');
        setCooldown(damagingEntity.id, 'sovereignmeteor');
    }

    if (Math.random() < 0.4 && !isOnCooldown(damagingEntity.id, 'ground_impact')) {
        damagingEntity.runCommand('function Movesets/Sovereignstyle/ground_impact');
        setCooldown(damagingEntity.id, 'ground_impact');
    }
});

system.runInterval(() => {
    const now = system.currentTick;

    if (now - lastTeleportTick >= TELEPORT_COOLDOWN) {
        const entities = world.getDimension('overworld').getEntities({ tags: ['mafangs_target'] });

        if (entities.length > 0) {
            const mafangs = world.getDimension('overworld').getEntities({ type: ENTITY_ID });

            mafangs.forEach((mafang) => {
                if (Math.random() < 0.5) {
                    teleportToTarget(mafang, entities[0]);
                    lastTeleportTick = now;
                }
            });
        }
    }
}, 20);

const playerTargets = new Map();

world.afterEvents.entityHitEntity.subscribe((data) => {
    const { damagingEntity, hitEntity } = data;

    if (damagingEntity.typeId !== "minecraft:player") return;

    const playerName = damagingEntity.nameTag.toLowerCase();
    const targetTag = `${playerName}_target`;

    // Remove a tag do alvo anterior, se existir
    const previousTarget = playerTargets.get(damagingEntity.id);
    if (previousTarget) {
        previousTarget.removeTag(targetTag);
    }

    // Define o novo alvo
    hitEntity.addTag(targetTag);
    playerTargets.set(damagingEntity.id, hitEntity);

    // Aplica efeitos
    damagingEntity.addEffect("slow_falling", 60, { amplifier: 0, showParticles: true });
    hitEntity.addEffect("slow_falling", 60, { amplifier: 0, showParticles: true });

    // Remove a tag do alvo após um tempo
    system.runTimeout(() => {
        hitEntity.removeTag(targetTag);
        if (playerTargets.get(damagingEntity.id) === hitEntity) {
            playerTargets.delete(damagingEntity.id); // Reseta se o alvo for o mesmo
        }
    }, 60);
});

// Reseta o sistema quando o alvo morre
world.afterEvents.entityDie.subscribe((data) => {
    const { deadEntity } = data;

    for (const [playerId, targetEntity] of playerTargets.entries()) {
        if (targetEntity.id === deadEntity.id) {
            const player = world.getEntity(playerId);
            if (player) {
                const playerName = player.nameTag.toLowerCase();
                const targetTag = `${playerName}_target`;

                deadEntity.removeTag(targetTag);
                playerTargets.delete(playerId);
            }
        }
    }
});

// Verifica teleporte a cada tick
system.runInterval(() => {
    for (const player of world.getPlayers()) {
        if (player.hasTag("teleport")) {
            const playerName = player.nameTag.toLowerCase();
            const targetTag = `${playerName}_target`;

            const targets = world.getDimension(player.dimension.id).getEntities({ tags: [targetTag] });

            if (targets.length > 0) {
                const target = targets[0];

                if (target.isValid()) {
                    player.teleport(target.location, { dimension: target.dimension });
                    player.runCommand(
                        `particle minecraft:portal ${player.location.x} ${player.location.y} ${player.location.z}`
                    );
                    player.runCommand(
                        `particle minecraft:portal ${target.location.x} ${target.location.y} ${target.location.z}`
                    );
                }
            } else {
                // Reseta o sistema se o alvo não existir
                playerTargets.delete(player.id);
            }

            player.removeTag("teleport");
        }
    }
}, 0);